/*-
 * Copyright (c) 1999-2002 Andrey Simonenko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *   @(#)$Id: rules.h,v 1.4.2.7 2002/09/24 10:52:30 simon Exp $
 */

#ifndef IPA_RULES_H
#define IPA_RULES_H


#include "cmd.h"
#include "kipfil.h"
#ifdef __FreeBSD__
# include "kip6fw.h"
#endif
#ifdef __OpenBSD__
# include "kpf.h"
#endif
#include "system.h"

#define ADD	  1
#define SUB	(-1)


#ifdef __FreeBSD__
/*
 * In original structure ip_fw there is only one field for rule number.
 * I add extra field subnumber: if there are some rules with the same number
 * I distinguish such rules by subnumbers, first rule in the set of such
 * rules has subnubner equal to 0. 
 */
struct ipfwac {
	u_short		number;		/* IPFW rule number */
	u_int		subnumber;	/* IPFW rule subnumber */
	short		set;		/* ==1 if this rule has been already
					   set in the IPFW kernel table */
	short		action;		/* subtract or add */
	u_quad_t	bcnt_old;	/* old value of fw_bcnt (struct ip_fw) */
};
#endif /* __FreeBSD__ */

#ifdef USE_OpenBSD_PF
struct pfac {
	u_int		number;		/* PF rule number */
	short		set;		/* ==1 if this rule has been already
					   set in the PF kernel table */
	short		action;		/* subtract or add */
	u_quad_t	bcnt_old;	/* old value of bytes (struct pf_rule) */
};
#endif /* USE_OpenBSD_PF */

#ifdef USE_IPFIL
struct ipfilac_list {
	u_int		number;		/* number in the group */
	short		set;		/* ==1 if this rule has been already
					   set in IPFIL kernel table */
	short		action;		/* subtract or add */
	u_quad_t	bcnt_old;	/* old value of fr_bytes (struct frentry) */
};

struct ipfilac {
	u_short		group;		/* the group number */
	char		type;		/* 'i' or 'o' */
	struct ipfilac_list *list;	/* list of IPFIL rules numbers in this
					   group of this type */
	u_int		nlist;		/* size of prev. array */
};
#endif /* USE_IPFIL */

#define UPTO_NOTSET	'-'
#define UPTO_SIMPLE	' '
#define UPTO_MINUTE	'm'
#define UPTO_HOUR	'h'
#define UPTO_dAY	'd'
#define UPTO_DAY	'D'
#define UPTO_wEEK	'w'
#define UPTO_WEEK	'W'
#define UPTO_MONTH	'M'

struct time_exp {
	time_t		seconds;	/* number of seconds */
	u_int		monthes;	/* number of monthes */
	char		upto;		/* m, h, D, W, M, - (not set) */
	short		side;		/* == 0 if +X is set on the left side,
					   == 1 if +X if set on the right side */
};

struct interval {
	u_short		h1, m1, h2, m2;	/* h1:m1 - h2:m2 */
};

struct db_group {
	int	group_set;		/* ==1 if "db_group" parameter  was used */
	u_long	group_id;		/* GID */
	int	group_named;		/* ==1 if group was given by name */
	mode_t	dir_mode;		/* mode of the database directory */
	mode_t	file_mode;		/* mode of the database file */
};

struct worktime {
	struct interval	*interval;	/* ptr. to interval list */
	u_int		ninterval;	/* size of prev. array */
	u_int		current;	/* current interval in *interval */
};

struct expire {
	struct time_exp	time;		/* "expire_time" parameter */
	struct cmd	*cmd;		/* "exec" parameters */
	u_int		ncmd;		/* size of prev. array */
};

struct reach {
	struct cmd	*cmd;		/* "exec" parameters */
	u_int		ncmd;		/* size of prev. array */
};

struct limit {
	FILE		*fp;		/* ptr. on FILE stream for limit db */
	int		fd;		/* == fileno(fp) */
	char		*filename;	/* name of limit database file */
	char		*limitname;	/* name of limit */
	u_quad_t	byte_limit;	/* limit { byte_limit } */
	u_quad_t	bcnt;		/* byte counter for this limit */
	u_quad_t	bcnt_sub;	/* number of bytes which should be subtracted
					   from limit counter, isn' equal to 0
					   if ``wrong'' configuration */
	struct reach	reach;		/* limit { reach {}} */
	struct time_exp	zero_time_param;/* limit { zero_time } */
	struct expire	expire;		/* limit { expire {}} */
	struct commands	rc[2];		/* rc[0] -- limit { startup {}},
					   rc[1] -- limit { shutdown {}} */
	char		*info;		/* limit { info } */
	pid_t		pid;		/* PID of background process which
					   runs "exec" parameter commands */
	u_short		is_changed;	/* ==1 if bcnt was modified */
	u_short		status;		/* status */
	time_t		wakeup;
	struct worktime	worktime[7];	/* limit { worktime } */
	short		use_worktime;	/* ==1 if "worktime" parameter was used */
	time_t		start_time;	/* time when limit was started */
	time_t		zero_time;	/* time when limit should be zeroed */
	time_t		reach_time;	/* time when limit was reached */
	time_t		expire_time;	/* time when limit will be expired */
};

#define	LIMIT_IS_REACHED	0x0001
#define LIMIT_EXEC		0x0002
#define LIMIT_EXPIRE_CHECKED	0x0004
#define LIMIT_EXPIRE_EXEC	0x0008


struct ipac {
	char		*rulename;	/* name of rule */
	u_int		update_db_time;	/* interval in sec. when to update counters,
					   rule { update_db_time } */
	u_int		append_db_time;	/* interval in sec. when to append new record in data base.
					   rule { append_db_time }*/
	time_t		wakeup;		/* time in sec. when to wake up */
	time_t		newrec_time;	/* time when to append new record in data base */
	FILE		*fp;		/* ptr. on FILE stream for db file */
	int		fd;		/* == fileno(fp) */
	char		*filename;	/* name of rule database file */
	u_quad_t	bcnt;		/* byte counter for this rule */
	u_quad_t	bcnt_sub;	/* nuber of bytes which should be subtracted
					   from accounting, isn't equal to 0
					   if ``wrong'' configuration */
	u_quad_t	maxchunk;	/* rule { maxchunk } */
#ifdef __FreeBSD__
	struct ipfwac	*ipfwac;	/* ptr. on array for IPFW accounting */
	u_int		nipfwac;	/* size of prev. array */
# ifdef USE_FreeBSD_IP6FW
	struct ipfwac	*ip6fwac;	/* ptr. on array for IP6FW accounting */
	u_int		nip6fwac;	/* size of prev. array */
# endif
#endif /* __FreeBSD__ */
#ifdef USE_OpenBSD_PF
	struct pfac	*pfac;		/* ptr. on array for PF accounting */
	u_int		npfac;		/* size of prev. array */
#endif /* USE_OpenBSD_PF */
#ifdef USE_IPFIL
	struct ipfilac	*ipfilac;	/* ptr. on array for IPFIL accounting */
	u_int		nipfilac;	/* size of prev. array */
#endif /* USE_IPFIL */
	struct limit	*limit;		/* ptr. on array of limits */
	u_int		nlimit;		/* size of prev. array */
	struct commands	rc[2];		/* rc[0] -- rule { startup {}},
					   rc[1] -- rule { shutdown {}} */
	struct db_group	db_group;	/* rule { db_group } */
	char		*info;		/* rule { info } */
	struct worktime	worktime[7];	/* rule { worktime } */
	short		use_worktime;	/* ==1 if "worktime" parameter is used */
};


extern struct ipac	*ipac;
extern u_int		nipac;
#ifdef __FreeBSD__
extern int		use_ipfw;
# ifdef USE_FreeBSD_IP6FW
extern int		use_ip6fw;
# endif
#endif /* __FreeBSD__ */
#ifdef USE_IPFIL
extern int		use_ipfil;
#endif /* USE_IPFIL */
#ifdef USE_OpenBSD_PF
extern int		use_pf;
#endif /* USE_OpenBSD_PF */
extern int		use_worktime_global;
extern struct worktime	worktime_global[7];
extern struct commands	startup_global, shutdown_global;

extern struct ipac	* ipac_new(void);
extern int		run_ipac(void);
extern void		end_work(int), sig_alrm(int), reconfigure(int),
			sig_chld(int), force_db_dump(int);
extern void		free_cmd_list(struct cmd *, u_int);
extern int		run_rules_rc(int);
extern int		init_limits(int);
extern int		free_ipac(struct ipac *);


#endif /* !IPA_RULES_H */
